<?php

require_once dirname(__FILE__) . str_replace('/', DIRECTORY_SEPARATOR, '/../../PrizmPayClient/PrizmPayClient.php');
require_once dirname(__FILE__) . str_replace('/', DIRECTORY_SEPARATOR, '/../../PrizmPayClient/PrizmPayClient_PP.php');

class Prizm_Prizmpay_Configs {

    public $testing;
    public $merchant_id;
    public $secret_key;

}

class Prizm_Prizmpay_Model_Prizmpay extends Mage_Payment_Model_Method_Abstract {

    protected $_code                   = 'prizmpay';
    protected $_isGateway              = true;
    protected $_canAuthorize           = true;
    protected $_canCapture             = true;
    protected $_canCapturePartial      = false;
    protected $_canRefund              = false;
    protected $_canVoid                = false;
    protected $_canUseInternal         = false;
    protected $_canUseCheckout         = true;
    protected $_canUseForMultishipping = false;
    protected $_canSaveCc              = false;
    protected $_formBlockType          = 'Prizm_Prizmpay_Block_Form';
    protected $_allowCurrencyCode      = array('HKD', 'USD', 'SGD', 'CNY', 'JPY', 'TWD', 'AUD', 'EUR', 'GBP', 'CAD', 'MOP', 'PHP', 'THB', 'MYR', 'IDR', 'KRW', 'SAR', 'NZD', 'AED', 'BND');

    /**
     * @var Prizm_Prizmpay_Configs 
     */
    protected $_configs      = NULL;
    protected $_configFields = array(
        'testing'     => 'Testing Mode',
        'merchant_id' => 'Merchant ID',
        'secret_key'  => 'Secret Key',
    );

    /**
     * @return Prizm_Prizmpay_Configs
     */
    protected function _loadConfigs() {
        if (is_null($this->_configs)) {
            $this->_configs = new Prizm_Prizmpay_Configs();
            foreach (array_keys($this->_configFields) as $field) {
                $this->_configs->$field = $this->getConfigData($field);
            }
        }
        return $this->_configs;
    }

    /**
     * @param Mage_Sales_Model_Order_Address $address
     */
    protected function _getAddressArray($address) {
        $result = array(
            'firstName'   => $address->getData('firstname'),
            'lastName'    => $address->getData('lastname'),
            'email'       => $address->getData('email'),
            'phoneNumber' => $address->getData('telephone'),
            'street1'     => $address->getStreet1(),
            'street2'     => $address->getStreet2(),
            'city'        => $address->getData('city'),
            'state'       => $address->getRegionCode(),
            'country'     => $address->getCountry(),
            'postalCode'  => $address->getData('postcode'),
        );
        return $result;
    }

    /**
     * Get session namespace
     *
     */
    public function getSession() {
        return Mage::getSingleton('prizmpay/prizmpay_session');
    }

    /**
     * Get checkout session namespace
     *
     * @return Mage_Checkout_Model_Session
     */
    public function getCheckout() {
        return Mage::getSingleton('checkout/session');
    }

    /**
     * Get current quote
     *
     * @return Mage_Sales_Model_Quote
     */
    public function getQuote() {
        return $this->getCheckout()->getQuote();
    }

    public function handleDatafeed($response = TRUE) {

        $this->_loadConfigs();

        if (!($data = PrizmPayClient::datafeed($this->_configs->merchant_id, $this->_configs->secret_key))) {
            if ($response) {
                echo 'ERROR';
            }
            exit;
        }

        $details       = json_decode($data['details'], TRUE);
        $merchantRef   = $data['client_ref'];
        $transactionID = $data['transaction_id'];
        $paymentStatus = $details['status'];

        if ($response) {
            echo 'PRIZMPAY';
        }

        if ($paymentStatus != 1) {
            return;
        }

        try {
            $comment = "Payment was Accepted. Transaction ID: " . $transactionID;

            /* @var $order Mage_Sales_Model_Order */
            $order     = Mage::getModel('sales/order')->loadByIncrementId($merchantRef);
            $orderData = $order->_data;
            if (isset($orderData['total_paid']) && $orderData['total_paid']) {
                return;
            }
            $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true, $comment, 1)->save();
            $order->sendNewOrderEmail();
            $order->sendOrderUpdateEmail(true, $comment);
            $payment = Mage::getSingleton('sales/order_payment')
                    ->setMethod('prizmpay')
                    ->setTransactionId($transactionID)
                    ->setIsTransactionClosed(true);
            $order->setPayment($payment);
            $payment->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_PAYMENT);
            $order->save();

            /* @var $invoice Mage_Sales_Model_Service_Order */
            $invoice = $order->prepareInvoice();
            $invoice->register()->capture();
            Mage::getModel('core/resource_transaction')
                    ->addObject($invoice)
                    ->addObject($invoice->getOrder())
                    ->save();
            $order->save();
        } catch (Mage_Core_Exception $e) {
            Mage::log($e);
            Mage::logException($e);
        }
    }

    /**
     * 
     * @param Mage_Sales_Model_Order $order
     */
    public function getCheckoutHTML($order) {

        $this->_loadConfigs();

        $gatewayURL = 'https://pp-test.prizmmpay.com';
        if ($this->_configs->testing == 0) {
            $gatewayURL = 'https://pp.prizmmpay.com';
        }

        $orderRef = $order->getData('increment_id');

        $pp = new PrizmPayClient_PP($gatewayURL, $this->_configs->merchant_id, $this->_configs->secret_key);
        $pp->setReferenceCode($orderRef);
        $pp->setInvoiceID($orderRef);

        $items = $order->getAllItems();
        if (isset($items) && is_array($items)) {
            foreach ($items AS $itemObj) {
                $item = $itemObj->_data;
                $pp->addItem($item['name'], (int) $item['qty_ordered'], $item['price']);
            }
        }
        $shippingCost = $order->getData('shipping_amount');
        if ($shippingCost > 0) {
            $pp->addItem($order->getData('shipping_description'), 1, $shippingCost);
        }
        $tax = $order->getData('base_tax_amount');
        if ($tax > 0) {
            $pp->addItem('Tax', 1, $tax);
        }

        $billingArray  = $this->_getAddressArray($order->getBillingAddress());
        $shippingArray = $this->_getAddressArray($order->getShippingAddress());

        if (!filter_var($shippingArray['email'], FILTER_VALIDATE_EMAIL)) {
            $shippingArray['email'] = $billingArray['email'];
        }
        if (!$billingArray['state']) {
            $billingArray['state'] = 'XX';
        }
        if (!$shippingArray['state']) {
            $shippingArray['state'] = 'XX';
        }

        $pp->setGrandTotal($order->getBaseGrandTotal(), $order->getBaseCurrency()->getCurrencyCode());
        $pp->setBillingAddressArray($billingArray);
        $pp->setShippingAddressArray($shippingArray);

        return $pp->html();
    }

    public function createFormBlock($name) {
        $block = $this->getLayout()->createBlock('prizmpay/prizmpay_form', $name)
                ->setMethod('prizmpay')
                ->setPayment($this->getPayment())
                ->setTemplate('prizm/prizmpay/form.phtml');

        return $block;
    }

    public function validate() {
        parent::validate();
        $currencyCode = $this->getQuote()->getBaseCurrencyCode();
        if ($currencyCode) {
            if (!in_array($currencyCode, $this->_allowCurrencyCode)) {
                Mage::throwException(Mage::helper('prizmpay')->__('Selected currency code (' . $currencyCode . ') is not compatabile with Prizm Pay'));
            }
        }
        return $this;
    }

    public function onOrderValidate(Mage_Sales_Model_Order_Payment $payment) {
        return $this;
    }

    public function onInvoiceCreate(Mage_Sales_Model_Invoice_Payment $payment) {
        return $this;
    }

    public function getOrderPlaceRedirectUrl() {
        return Mage::getUrl('prizmpay/prizmpay/redirect');
    }

}
